/*
*******************************************************************************
*
*          CONFIDENTIALITY AND COPYRIGHT NOTICE
*
*******************************************************************************
*
* THIS DOCUMENT CONSTITUTES VALUABLE CONFIDENTIAL AND PROPRIETARY INFORMATION.
* ANY UNAUTHORIZED USE, REPRODUCTION OR TRANSFER OF THE PROGRAM OR
* ACCOMPANYING FILES, IN WHOLE OR IN PART, IS A VIOLATION OF APPLICABLE LAWS.
*
* This copyrighted work is subject to limited distribution and restricted
* disclosure only.  Unless required by applicable law or agreed to in writing,
* this information is provided on an "AS IS" BASIS WITHOUT WARRANTIES OF ANY
* KIND EITHER EXPRESS OR IMPLIED.
*
* Use of this software is governed by the End-User License Agreement
* distributed with this Material.
*
* Copyright © 2006-2017 LineStream Technologies, Inc., ALL RIGHTS RESERVED.
* Protected under U.S., Foreign, and Pending Patents as described here:
*    http://www.linestream.com/patents
*
******************************************************************************/
/**
* @file        drv.c
* @brief       Drive (DRV) module C file Sensorless Velocity - Hall Start
*/
/**
* \addtogroup DRVSNLESSHALL
* @{
*/


/*
 * Include
 */
#include <assert.h>

#include <stdint.h>
#include "string.h"
#include "qmath.h"
#include "trf.h"
#include "flt.h"
#include "drv.h"
#include "dsm.h"
#include "sysparams.h"
#include "speed.h"
#include "current.h"
#include "fw.h"
#include "svpwm.h"
#include "clarke.h"
#include "park.h"
#include "ipark.h"
#include "startup.h"
#include "est.h"
#include "trajvel.h"
#include "user.h"
#include "hall.h"

#include "system.h"

#include "MotionSequence.h"

#pragma diag_suppress=Ta022,Ta023


/*
 * Global variables
 */

/**
 * @def     FLT_PLL_OUTPUT_CUTOFF_FREQUENCY
 * @brief   Cut-off frequency for filtering PLL speed
 */
#define FLT_PLL_OUTPUT_CUTOFF_FREQUENCY  (21.605880483258250)

/**
 * @def     FLT_SPEED_CUTOFF_1MS_FREQUENCY
 * @brief   Cut-off frequency for filtering rotor speed
 */
#define FLT_SPEED_CUTOFF_1MS_FREQUENCY  (50.0)

/**
 * @def     FLT_SPEED_CUTOFF_HALLS_FREQUENCY
 * @brief   Cut-off frequency for filtering rotor speed derived from HALLS
 */
#define FLT_SPEED_CUTOFF_HALLS_FREQUENCY  (10.0)

/**
 * @def     FLT_SPEED_ERROR_CUTOFF_HALLS_FREQUENCY
 * @brief   Cut-off frequency for filtering rotor speed error derived from HALLS
 */
#define FLT_SPEED_ERROR_CUTOFF_HALLS_FREQUENCY  (10.0)

/**
 * @def     FLT_POWER_CUTOFF_1MS_FREQUENCY
 * @brief   Cut-off frequency for filtering powers
 */
#define FLT_POWER_CUTOFF_1MS_FREQUENCY  (1.0)

/**
 * @def     FLT_ISDQ_CUTOFF_1MS_FREQUENCY
 * @brief   Cut-off frequency for filtering Isdq currents
 */
#define FLT_ISDQ_CUTOFF_1MS_FREQUENCY   (0.1)

/**
 * @def     FLT_FLUX_CUTOFF_1MS_FREQUENCY
 * @brief   Cut-off frequency for filtering stator flux space vector magnitude
 */
#define FLT_FLUX_CUTOFF_1MS_FREQUENCY  (10.0F)

/**
 * @def     FLT_VDC_CUTOFF_PWM_FREQUENCY
 * @brief   Cut-off frequency for filtering DC bus voltage
 */
#define FLT_VDC_CUTOFF_FREQUENCY_1kHz    (1000.0F)

/**
 * @def     FLT_VDC_CUTOFF_FREQUENCY_200Hz
 * @brief   Cut-off frequency for filtering DC bus voltage
 */
#define FLT_VDC_CUTOFF_FREQUENCY_200Hz   (200.0F)

/**
 * @def     FLT_VDC_CUTOFF_FREQUENCY_1Hz
 * @brief   Cut-off frequency for filtering DC bus voltage
 */
#define FLT_VDC_CUTOFF_FREQUENCY_1Hz   (1.0F)

/**
 * @def     FLT_RMS_CUTOFF_FREQUENCY
 * @brief   Cut-off frequency for filtering of RMS values
 */
#define FLT_RMS_CUTOFF_FREQUENCY        (10.0F)

/**
 * @def     FLT_ID_REF_CUTOFF_FREQUENCY
 * @brief   Cut-off frequency for filtering FW Id reference
 */
#define FLT_ID_REF_CUTOFF_FREQUENCY (0.1F)

/**
 * @def     FLT_ANG_DIFF_CUTOFF_FREQUENCY
 * @brief   Cut-off frequency for filtering angle difference
 */
#define FLT_ANG_DIFF_CUTOFF_FREQUENCY (1.0F)

/**
 * @def     FLT_ANG_OFFSET_CUTOFF_FREQUENCY
 * @brief   Cut-off frequency for filtering angle offset
 */
#define FLT_ANG_OFFSET_CUTOFF_FREQUENCY (30.0F)

/**
 * @def     FLT_IDQ_ERROR_CUTOFF_FREQUENCY
 * @brief   Cut-off frequency for filtering current error
 */
#define FLT_IDQ_ERROR_CUTOFF_FREQUENCY  (5000.0F)

/**
 * @var       speed
 * @brief     SPEED regulator block
 */
static SPEED_t speed;

/**
 * @var       current
 * @brief     CURRENT regulator block
 */
static CURRENT_t current;

/**
 * @var       fw
 * @brief     FW regulator block
 */
static FW_t fw;

/**
 * @var       feedback
 * @brief     FEEDBACK block
 */
static FEEDBACK_t feedback;

/**
 * @var       svpwm
 * @brief     SVPWM block
 */
static SVPWM_t svpwm;

/**
 * @var       clarke
 * @brief     CLARKE block
 */
static CLARKE_t clarke;

/**
 * @var       park
 * @brief     PARK block
 */
static PARK_t park;

/**
 * @var       ipark
 * @brief     IPARK block
 */
static IPARK_t ipark;

/**
 * @var       startup
 * @brief     STARTUP block
 */
static STARTUP_t startup;

/**
 * @var       est
 * @brief     EST block
 */
static EST_t est;

/**
 * @var       trajvel
 * @brief     TRAJVEL block
 */
static TRAJVEL_t trajvel;

/**
 * @var       trajvelTEST
 * @brief     TRAJVEL block for testing profile
 */
static TRAJVEL_t trajvelTEST;

/**
 * @var       trajvelTESTEnable
 * @brief     Signal to run the test instance of the TRAJVEL block
 */
static bool trajvelTESTEnable = false;

/**
 * @var       hall
 * @brief     HALL block
 */
static HALL_t hall;

/**
 * @var       dsm
 * @brief     DSM block
 */
static DSM_t dsm;

/**
 * @var       user
 * @brief     USER block
 */
USER_t user; /* Note:  not static since it is intented to be application layer interface */

/**
 * @fn        void DRV_updateAtFastTick(void)
 * @brief     Updates DRV module at PWM tick
 * @return    none
 */
static inline  FAST void DRV_updateAtFastTick(void);

/**
 * @fn        bool DRV_isDCInjectionEnabled(USER_t *pUSER)
 * @brief     Checks components that might want to do DC Injection and returns if DC Injection should be done
 * @return    true if DC Injection should be enabled, false is DC Injection should not be enabled
 */
static FAST bool DRV_isDCInjectionEnabled(USER_t *pUSER);

/**
 * @fn        bool DRV_isSpeedLoopEnabled(DSM_t *pDSM)
 * @brief     Checks components that might want to run speed loop
 * @return    true if speed loop should be enabled, false is speed loop should not be enabled
 */
static FAST bool DRV_isSpeedLoopEnabled(DSM_t *pDSM);

/**
 * @fn        bool DRV_isTrajectoryEnabled(USER_t *pUSER)
 * @brief     Checks components that might want to enable the trajectory module
 * @return    true if trajectory loop should be enabled, false if trajectory should not be enabled
 */
static FAST bool DRV_isTrajectoryEnabled(USER_t *pUSER);

/**
 * @fn        bool DRV_isTrajectoryIdle(TRAJVEL_t *traj)
 * @brief     Checks components that might want to enable the trajectory module
 * @return    true if trajectory loop should be enabled, false if trajectory should not be enabled
 */
static bool DRV_isTrajectoryIdle(TRAJVEL_t *pTRAJ);

/**
 * @fn        bool DRV_isFieldWeakeningEnabled(USER_t *pUSER)
 * @brief     Checks components that might want to disable field weakening
 * @return    true if field loop should be enabled, false is field loop should not be enabled
 */
inline static FAST  bool DRV_isFieldWeakeningEnabled(USER_t *pUSER);

/**
 * @fn        void DRV_init(const SystemParams_t* const flashSysParams)
 * @brief     Initializes DRV module
 * @return    none
 */

void DRV_init(const SystemParams_t * const flashSysParams)
{

    DSM_init(&dsm, sizeof(dsm));

    FEEDBACK_init(&feedback, sizeof(feedback));
    FEEDBACK_config(&feedback, &flashSysParams->gains, flashSysParams->focSampleTime,
                    _LQ16(FLT_VDC_CUTOFF_FREQUENCY_1Hz), _LQ16(FLT_VDC_CUTOFF_FREQUENCY_200Hz), _LQ16(FLT_VDC_CUTOFF_FREQUENCY_1kHz), _LQ16(FLT_RMS_CUTOFF_FREQUENCY));

    SVPWM_init(&svpwm, sizeof(svpwm));
    SVPWM_config(&svpwm, &flashSysParams->svpwmCfg, flashSysParams->pwmFreqHz, flashSysParams->deadTimePerTs, flashSysParams->enableHighSideActiveLow, flashSysParams->enableLowSideActiveLow);

    /* Initialize DSM configuration bits */
    DSM_configFaults(&dsm, &(flashSysParams->dsmFaultCfg));

    TRAJVEL_init(&trajvel, sizeof(trajvel));
    TRAJVEL_config(&trajvel, &flashSysParams->trajvelCfg, flashSysParams->lq20HaltJerkLimit, flashSysParams->slowSampleTime);

    /* this block is only required so that profiles can be tested in KMS */
    TRAJVEL_init(&trajvelTEST, sizeof(trajvelTEST));
    TRAJVEL_config(&trajvelTEST, &flashSysParams->trajvelCfg, flashSysParams->lq20HaltJerkLimit, flashSysParams->slowSampleTime);
    trajvelTEST.config.test = true;
    trajvelTESTEnable = false;

    SPEED_init(&speed, sizeof(speed));
    SPEED_config(&speed, &flashSysParams->speedCfg, flashSysParams->slowSampleTime, flashSysParams->motorParams.maxStatorCurrent, flashSysParams->motorParams.pmFlux);

    CURRENT_init(&current, sizeof(current));
    CURRENT_config(&current, &flashSysParams->currentCfg, flashSysParams->focSampleTime, _LQ16(FLT_IDQ_ERROR_CUTOFF_FREQUENCY), flashSysParams->motorParams.voltageGain);

    FW_init(&fw, sizeof(fw));
    FW_config(&fw, &flashSysParams->fwCfg, flashSysParams->slowSampleTime, _LQ16(FLT_ID_REF_CUTOFF_FREQUENCY));

    CLARKE_init(&clarke, sizeof(clarke));
    PARK_init(&park, sizeof(park));
    IPARK_init(&ipark, sizeof(ipark));

    STARTUP_init(&startup, sizeof(startup));
    STARTUP_config(&startup, _LQsinPU, &flashSysParams->startupCfg, flashSysParams->lq16BaseSpeedElecFreq, flashSysParams->focSampleTime, _LQ16(FLT_ANG_DIFF_CUTOFF_FREQUENCY));

    QEIS_initHallInputs();

    HALL_init(&hall, sizeof(hall));
    HALL_config(&hall, &flashSysParams->hallCfg, flashSysParams->lq16BaseSpeedElecFreq, flashSysParams->focSampleTime, flashSysParams->motorParams.polePairs, _LQ16(FLT_SPEED_CUTOFF_HALLS_FREQUENCY), _LQ16(FLT_SPEED_ERROR_CUTOFF_HALLS_FREQUENCY));

    EST_init(&est, sizeof(est));
    EST_config(&est, &flashSysParams->estCfg, flashSysParams->lq16BaseSpeedElecFreq, flashSysParams->focSampleTime, flashSysParams->slowSampleTime, flashSysParams->deadTimePerTs,
               _LQ16(FLT_SPEED_CUTOFF_1MS_FREQUENCY), _LQ16(FLT_POWER_CUTOFF_1MS_FREQUENCY), _LQ16(FLT_ISDQ_CUTOFF_1MS_FREQUENCY), _LQ16(FLT_FLUX_CUTOFF_1MS_FREQUENCY), _LQ16(FLT_ANG_OFFSET_CUTOFF_FREQUENCY));

    /* User init module */
    USER_init(&user, sizeof(user));

    /* User init brake, SCM, Inertia, and Plan */
    USER_initStructs(&user, flashSysParams->trajvelCfg.limitAcc, flashSysParams->trajvelCfg.lq20LimitJerk, flashSysParams->speedCfg.IqRefMax, flashSysParams->speedCfg.IqRefMin);

    /* User configuration */
    USER_config(&user, &flashSysParams->userCfg);

    /* User inertia config */
    USER_cfgInertia(&flashSysParams->inertiaCfg, flashSysParams->slowSampleTime);

    /* User SCM config (must be done after EST config) */
    USER_cfgSCM(&flashSysParams->scmCfg, est.output.SCM_data, flashSysParams->motorParams.ratedCurrent, flashSysParams->motorParams.ratedSpeed,
                flashSysParams->lq16BaseSpeedElecFreq, flashSysParams->focSampleTime);

    /* User brake config */
    USER_cfgBrake(&flashSysParams->brakeCfg);

} /* end of DRV_init() */

/**
 * @fn        void DRV_updateSlowTick(void)
 * @brief     Updates DRV module at slow tick
 * @return    none
 */
void DRV_updateSlowTick(void)
{
    static uint16_t slowTickCount = 0U;
    HALL_values_t hallReading;

    /* START USER BLOCK *************************************************************/
    if((user.state == USER_RUN_CURRENT) && (startup.output.controlType != FOC))
    {
        /* if running in torque mode, set a target speed to begin startup */
        /* make sure that the target speed is the same direction as the current */
        /* set the target speed to be equal to the FOC crossover speed */
        if(user.command.statorRefCurrent.q > _SQ(0.0))
        {
            user.command.targetSpeed = est.config.FOCLowSpeed;
        }
        else if(user.command.statorRefCurrent.q < _SQ(0.0))
        {
            user.command.targetSpeed = -est.config.FOCLowSpeed;
        }
        else
        {
            user.command.targetSpeed = _LQ(0.0);
        }
    }

    USER_updateSlowTick(&user, &dsm, DRV_isTrajectoryIdle(&trajvel), startup.output.controlType,  \
                        est.output.rotorSpeed_50Hz, startup.output.fieldAngle, trajvel.output.refSpeed);
    /* END USER BLOCK ***************************************************************/

    /* START EST BLOCK **************************************************************/
    EST_updateSlowTick(&est, user.output.runEstimator, trajvel.output.refSpeed,    \
                       feedback.output.invVdc_1kHz, DRV_isDCInjectionEnabled(&user));
    /* END EST BLOCK ****************************************************************/

    /* START TRAJVEL BLOCK ************************************************************/
    /* Accept trajectory commands from the USER */
    trajvel.config.targetSpeed = user.output.drvCommand.targetSpeed;
    trajvel.config.limitAcc = user.output.drvCommand.limitAcc;
    trajvel.config.lq20LimitJerk = user.output.drvCommand.lq20LimitJerk;

    TRAJVEL_run(&trajvel, DRV_isTrajectoryEnabled(&user));
    TRAJVEL_run(&trajvelTEST, trajvelTESTEnable);  /* This handles running the profile tester */
    if((trajvelTESTEnable == true) && (trajvelTEST.output.status == TRAJVEL_IDLE))
    {
        /* if trajectory is completed disable enable */
        trajvelTESTEnable = false;
    }
    /* END TRAJVEL BLOCK **************************************************************/

    /* START DSM BLOCK *************************************************************/
    DSM_updateSlowTick(&dsm, user.output.dsmCommand, &(feedback.output),         \
                       startup.output.controlType, trajvel.output.refSpeed,         \
                       est.output.rotorSpeed_50Hz, DRV_isSpeedLoopEnabled(&dsm), \
                       est.output.statorFluxMagnitude, user.output.disableFaults);
    /* END DSM BLOCK ***************************************************************/

    if(startup.output.controlType == FOC)
    {
#pragma diag_suppress=Pm057
        switch(dsm.state)
        {
        case DSM_MOTION:
            /* START FW BLOCK **************************************************************/
            FW_run(&fw, DRV_isFieldWeakeningEnabled(&user), est.output.statorRefVoltageMagnitude, \
                   trajvel.output.refSpeed, trajvel.config.targetSpeed);
            /* END FW BLOCK ****************************************************************/

            /* START SPEED BLOCK ***********************************************************/
            /* get updated max and min Iq references from USER */
            speed.config.IqRefMax = user.output.drvCommand.IqRefMax;
            speed.config.IqRefMin = user.output.drvCommand.IqRefMin;

            SPEED_run(&speed, DRV_isSpeedLoopEnabled(&dsm), trajvel.output.refSpeed, \
                      trajvel.output.refAcc, est.output.rotorSpeed, fw.output.IdRef);
            /* END SPEED BLOCK *************************************************************/
            break;

        case DSM_IDLE:  /* Intentional Fallthrough */
        case DSM_FAULT: /* Intentional Fallthrough */
            /* Keep ramp command set to 0 */
            TRAJVEL_reset(&trajvel, _LQ(0.0));

            /* reset startup and est module */
            STARTUP_reset(&startup);
            EST_reset(&est);

        /* Intentional Fallthrough */
        default: /* drive is running, but not speed mode */
            /* Flux-weakening loop inactive, hence flush the integrator */
            FW_resetController(&fw);
            /* Speed loop inactive, since the inverter is switched off */
            SPEED_reset(&speed);
            break;
        }
#pragma diag_default=Pm057

    }
    else /* startup.output.controlType == STARTUP */
    {
        /* START SPEED BLOCK ***********************************************************/
        /* force possible output to maximum */
        speed.config.IqRefMax = user.output.drvCommand.IqRefMax;
        speed.config.IqRefMin = user.output.drvCommand.IqRefMin;

        SPEED_run(&speed, true, trajvel.output.refSpeed, \
                  _LQ(0.0f), hall.output.rotorSpeed, _SQ(0.0));

        if( trajvel.output.refSpeed >= _LQ(0.0) && speed.output.IqRef < _SQ15(0.0f) )
        {
            SPEED_set(&speed, _SQ15(0.0f), hall.output.rotorSpeed);
            speed.output.IqRef = _SQ15(0.0f);
        }
        else if (trajvel.output.refSpeed < _LQ(0.0) && speed.output.IqRef > -_SQ15(0.0f))
        {
            SPEED_set(&speed, -_SQ15(0.0f), hall.output.rotorSpeed);
            speed.output.IqRef = -_SQ15(0.0f);
        }
        /* END SPEED BLOCK *************************************************************/

        if (dsm.state >= DSM_CURRENT)
        {
            /* START HALL BLOCK ************************************************************/
            if(user.state == USER_SCM)
            {
                /* Read Hall-effect sensors */
                QEIS_readHallInputs(&hallReading);
                /* only run the Hall capture when doing Flux Id */
                HALL_runIdentify(&hall, &hallReading, est.output.rotorSpeed, startup.output.fieldAngle);
            }
            /* END HALL BLOCK *************************************************************/

            /* START STARTUP BLOCK *********************************************************/
            STARTUP_updateSlowTick(&startup, trajvel.output.refSpeed, est.output.statorFluxMagnitude);
            /* END STARTUP BLOCK ***********************************************************/

            /* Flux-weakening loop inactive */
            FW_resetController(&fw);
        }
        else
        {
            /* Keep ramp command set to 0 */
            TRAJVEL_reset(&trajvel, _LQ(0.0));

            /* Flux-weakening loop inactive */
            FW_resetController(&fw);

            /* Reset speed loop */
            SPEED_reset(&speed);

            /* reset startup and est module */
            STARTUP_reset(&startup);
            EST_reset(&est);
        }
    }

    slowTickCount++;
} /* end of DRV_updateSlowTick() */

/**
 * @fn        void DRV_updateAtFastTick(void)
 * @brief     Updates DRV module at PWM tick
 * @return    none
 */
inline static void DRV_updateAtFastTick(void)
{
    static DRV_control_e controlTypeOld = STARTUP;
    static TRF_orient32_t tempOrient = {_LQ(0.0),_LQ(1.0)};

    _lq tempAngle = _LQ(0.0);
    TRF_rVector32_t tempStatorRefVoltageDQ; /* the stator reference voltage */
    TRF_rVector16_t tempStatorRefCurrentDQ; /* the stator reference current  */
    TRF_rVector16_t tempStatorFdbCurrentDQ; /* the stator feedback current  */
    HALL_values_t hallReading;

    /* Read USER commands to update DRV and DSM variables */
    svpwm.config.enableClampPhases = user.output.drvCommand.enableClampPhases;

    /* START DRIVE LOGIC BLOCK *****************************************************/
    if((dsm.state >= DSM_PWM_DUTY) && (user.output.drvCommand.pwmDisable == false))
    {
        /* Turn inverter switches on */
        SVPWM_enable(&svpwm);
    }
    else /* DSM_FAULT / DSM_IDLE / DSM_START / USER.drvCommand.pwmDisable */
    {
        /* Keep inverter switches off */
        SVPWM_disable(&svpwm);
    }
    /* END DRIVE LOGIC BLOCK *******************************************************/

    /* START CLARKE BLOCK **********************************************************/
    CLARKE_run(&clarke, &(feedback.output));
    /* END CLARKE BLOCK ************************************************************/

    /* START PARK BLOCK ************************************************************/
    PARK_run(&park, &tempOrient, &(clarke.output.vector));
    /* END PARK BLOCK **************************************************************/

    /* EST BLOCK *******************************************************************/
    EST_updateFastTick(&est, &(svpwm.output), &(feedback.output), &(clarke.output.vector), \
                       ipark.output.magnitude, &(park.output.vector));
    /* EST BLOCK *******************************************************************/

    /* START HALL BLOCK ***********************************************************/
    QEIS_readHallInputs(&hallReading);

    if(user.state != USER_SCM)
    {
        HALL_run(&hall, &hallReading, trajvel.output.refSpeed, &(feedback.output));
    }
    /* END HALL BLOCK *************************************************************/

    /* START STARTUP BLOCK *********************************************************/
    bool tempTest = false;

    if((hall.output.isTracking) && (startup.output.controlType == STARTUP) && (_LQabs(trajvel.output.refSpeed) >= est.config.FOCLowSpeed) )
    {
        tempTest = true;
    }
    else if(startup.output.controlType == FOC)
    {
    	if(_LQabs(trajvel.output.refSpeed) >= est.config.FOCLowSpeed)
    	{
    		tempTest = true;
    	}
    	else
    	{
    		tempTest = false;
    	}
    }

    STARTUP_updateFastTick(&startup, est.output.rotorAngle, tempTest, \
                           trajvel.output.refSpeed, park.output.vector.q);

    /* END STARTUP BLOCK ***********************************************************/

    /* START GLUE LOGIC ************************************************************/
    /* angle choice */
    if(startup.output.controlType == FOC)
    {
        tempAngle = est.output.rotorAngle;
    }
    else
    {
        if(user.state >= USER_RUN_DUTY)
        {
            tempAngle = hall.output.rotorAngle;
        }
        else
        {
            tempAngle = startup.output.fieldAngle;
        }
    }

    if(DRV_isDCInjectionEnabled(&user))
    {
        tempAngle = 0;
    }
    /* END GLUE LOGIC **************************************************************/

    _LQsincosPU(tempAngle, &tempOrient.sinAngle, &tempOrient.cosAngle);

    if((startup.output.controlType == FOC) && (controlTypeOld == STARTUP))
    {
        /* if we went into FOC, force the INERTIA output */
        USER_setInertiaOutput(&user, startup.output.statorRefCurrent.q);

        if(user.state >= USER_RUN_DUTY)
        {
            PARK_t tempPark;
            TRF_sVector32_t* tempStatorVoltageAB;
            TRF_sVector16_t  tempStatorVoltage16;
            TRF_rVector32_t tempStatorVoltageDQ;

            /* when we transition into FOC from TRAP, update the CURRENT integrators */
            tempStatorVoltageAB = EST_getStatorVoltageAB(&est);
            tempStatorVoltage16.A = _LQtoSQ15(tempStatorVoltageAB->A);
            tempStatorVoltage16.B = _LQtoSQ15(tempStatorVoltageAB->B);
            PARK_run(&tempPark, &tempOrient, &tempStatorVoltage16);
            tempStatorVoltageDQ.d = _SQ15toLQ(tempPark.output.vector.d);
            tempStatorVoltageDQ.q = _SQ15toLQ(tempPark.output.vector.q);
            CURRENT_resetController(&current);
            CURRENT_setIntegrator(&current, feedback.output.Vdc_1kHz, &tempStatorVoltageDQ);
        }
    }

    USER_updateFastTick(&user, &feedback.output, speed.config.IqRefMax, speed.config.IqRefMin);

    DSM_updateAtFastTick(&dsm, &feedback.output, user.output.disableFaults);

#pragma diag_suppress=Pm057

    switch(dsm.state)
    {
    case DSM_START:      /**< DSM start state */
    /* Intentional Fallthrough */
    case DSM_IDLE:       /**< DSM idle state */
    /* Intentional Fallthrough */
    case DSM_FAULT:      /**< DSM fault state */
        /* reset current controller */
        CURRENT_resetController(&current);
        /* reset IPARK block */
        IPARK_reset(&ipark);
        break;

    case DSM_MOTION:       /**< DSM speed control state */
        tempStatorRefCurrentDQ.d = fw.output.IdRef;
        tempStatorRefCurrentDQ.q = speed.output.IqRef;

    /* Intentional Fallthrough */

    case DSM_CURRENT:    /**< DSM current control state */
        /* choose who is writing the current reference */
        if(dsm.state == DSM_CURRENT)
        {
            tempStatorRefCurrentDQ = user.output.drvCommand.statorRefCurrent;
        }

        if((startup.output.controlType == STARTUP) && (DRV_isDCInjectionEnabled(&user) == false) && (user.state < USER_RUN_DUTY))
        { /* used to retain ability to perform SCM and INERTIA tests */
            tempStatorRefCurrentDQ = startup.output.statorRefCurrent;
        }

        /* choose who is writing the current feedback */
        tempStatorFdbCurrentDQ = park.output.vector;

        if((startup.output.controlType == STARTUP) && (DRV_isDCInjectionEnabled(&user) == false) && (user.state >= USER_RUN_DUTY))
        {
            /* moved here to simplify logic */
            /* used to handle startup when running in torque mode */
            tempStatorRefCurrentDQ.d = _SQ15(0.0);
            tempStatorRefCurrentDQ.q = speed.output.IqRef;
            tempStatorFdbCurrentDQ.d = _SQ15(0.0);
            tempStatorFdbCurrentDQ.q = hall.output.statorFdbCurrent;
        }

        /* START CURRENT BLOCK *********************************************************/
        CURRENT_run(&current, feedback.output.Vdc_1kHz,\
                    feedback.output.invVdc_1kHz, &tempStatorRefCurrentDQ, &tempStatorFdbCurrentDQ);
        /* END CURRENT BLOCK ***********************************************************/

        tempStatorRefVoltageDQ = current.output.statorRefVoltageDQ;

    /* Intentional Fallthrough */

    case DSM_VOLTAGE:    /**< DSM voltage control state */
        /* START GLUE LOGIC ************************************************************/
        /* choose who has the voltage reference */
        if(dsm.state == DSM_VOLTAGE)
        {
            tempStatorRefVoltageDQ = user.output.drvCommand.statorRefVoltage;
        }
        /* END GLUE LOGIC **************************************************************/

        /* START PARK BLOCK ************************************************************/
        IPARK_run(&ipark, &tempOrient, &tempStatorRefVoltageDQ);
    /* END PARK BLOCK **************************************************************/

    /* Intentional Fallthrough */

    case DSM_PWM_DUTY:   /**< DSM PWM duty control state */

    /* Intentional Fallthrough */

    default:
        break;
    }
#pragma diag_default=Pm057

    /* START SVPWM BLOCK ***********************************************************/
    if((startup.output.controlType == STARTUP) && (DRV_isDCInjectionEnabled(&user) == false) && (user.state >= USER_RUN_DUTY)) /* only use HALL Start when not running an identification */
    {
        SVPWM_update2Phase(&svpwm, hall.output.hallIndex, current.output.statorRefVoltageDQ.q);
    }
    else
    {
        SVPWM_updateFastTick(&svpwm, &(ipark.output.vector), ipark.output.magnitude);
    }
    /* END SVPWM BLOCK *************************************************************/

    /* Update the control type */
    controlTypeOld = startup.output.controlType;


} /* end of DRV_updateAtFastTick() */

/**
 * @fn        bool DRV_isDCInjectionEnabled(USER_t *pUSER)
 * @brief     Checks components that might want to do DC Injection and returns if DC Injection should be done
 * @return    true if DC Injection should be enabled, false is DC Injection should not be enabled
 */
static bool DRV_isDCInjectionEnabled(USER_t *pUSER)
{
    bool enable = false;

    if(pUSER->output.drvCommand.enableDCInjection == true)
    {
        enable = true;
    }

    return enable;
} /* end of DRV_isDCInjectionEnabled */



/**
 * @fn        bool DRV_isSpeedLoopEnabled(DSM_t *pDSM)
 * @brief     Checks components that might want to disable the speed loop
 * @return    true if speed loop should be enabled, false is speed loop should not be enabled
 */
static bool DRV_isSpeedLoopEnabled(DSM_t *pDSM)
{
    bool enable = true;

    if(pDSM->state != DSM_MOTION)
    {
        enable = false;
    }

    return enable;
}

/**
 * @fn        bool DRV_isTrajectoryEnabled(USER_t *pUSER)
 * @brief     Checks components that might want to enable the trajectory module
 * @return    true if trajectory loop should be enabled, false if trajectory should not be enabled
 */
static bool DRV_isTrajectoryEnabled(USER_t *pUSER)
{
    bool enable = false;

    if(pUSER->output.drvCommand.runTrajectory == true)
    {
        enable = true;
    }

    return enable;
}

/**
 * @fn        bool DRV_isTrajectoryIdle(TRAJVEL_t *pTRAJ)
 * @brief     Checks components that might want to enable the trajectory module
 * @return    true if trajectory loop should be enabled, false if trajectory should not be enabled
 */
static bool DRV_isTrajectoryIdle(TRAJVEL_t *pTRAJ)
{
    bool idle = false;

    if(pTRAJ->output.status == TRAJVEL_IDLE)
    {
        idle = true;
    }

    return idle;
}

/**
 * @fn        bool DRV_isFieldWeakeningEnabled(USER_t *pUSER)
 * @brief     Checks components that might want to disable field weakening
 * @return    true if speed loop should be enabled, false is speed loop should not be enabled
 */
static bool DRV_isFieldWeakeningEnabled(USER_t *pUSER )
{
    bool fwEnableReturn = true;
    if(pUSER->config.FWEnabled == false)
    {
        fwEnableReturn = false;
    }

    return fwEnableReturn;
}

/**
 * @fn      void UpdateAtFastTick(void)
 * @brief   ADC ISR function
 * @return  none
 */
#if __GNUC__
void UpdateAtFastTick(const ADC_results_t* const adc_results) __attribute__ ((section (".fast")));
#endif
void UpdateAtFastTick(const ADC_results_t* const adc_results)
{
    FEEDBACK_run(&feedback, svpwm.output.state, svpwm.output.sector, adc_results);

    /* Call only if ADC has finished calibration and is in RUN */
    switch (feedback.state)
    {
    case FEEDBACK_RUN:
        /* Update Drive module at fast tick */
        DRV_updateAtFastTick();
        break;

    case FEEDBACK_CALIB:
        break;

    case FEEDBACK_FAULT:
        dsm.nonClearableFaults.bits.calibrationFault = 1;
        break;

    default:
        assert(false);
        break;
    }
} /* end of UpdateAtFastTick() */

/**
* @fn          _lq DRV_getDcbBusVoltage(void)
* @brief      Return instancaneous sample of VDC Bus
* @return      _lq DC Bus Voltage { unit: [pu volts] }
*/
_lq DRV_getDcbBusVoltage(void)
{
    return feedback.output.Vdc;
}

/**
* @fn          _lq DRV_getDcbBusUnderVoltageThreshold(void)
* @brief        DC Bus Under Voltage Threshold
* @return      _lq Under Voltage Threshold Voltage { unit: [pu volts] }
*/
_lq DRV_getDcbBusUnderVoltageThreshold(void)
{
    /* DC BUS Undervoltage Fault Threshold */
    return dsm.faultThresholds.underVoltage;
}

/**
 * @fn          bool DRV_getIsFaultActive(void)
 * @brief       Any clearable or non-clearable fault is active
 */
bool DRV_getIsFaultActive(void)
{
    return (((dsm.clearableFaults.all != 0UL) || (dsm.nonClearableFaults.all != 0U)) && (dsm.faultThresholds.enableFaults));
}

/**
 * @fn          void DRV_forceToIdle(void)
 * @brief       Helper function to force control state to IDLE
 */
void DRV_forceToIdle(void)
{
    user.state = USER_IDLE;
}

/** @}*/ /*end of DRVSNLESSHALL */

/* [EOF] */
